@@ -6,6 +6,7 @@ from infrahub_sdk.protocols import (
     CoreNode,  # noqa: TC002
     CoreTransformPython,
 )
+from infrahub_sdk.template import Jinja2Template
 from prefect import flow
 from prefect.client.orchestration import get_client
 from prefect.logging import get_run_logger
@@ -16,7 +17,6 @@ from infrahub.core.registry import registry
 from infrahub.events import BranchDeletedEvent
 from infrahub.git.repository import get_initialized_repo
 from infrahub.services import InfrahubServices  # noqa: TC001  needed for prefect flow
-from infrahub.support.macro import MacroDefinition
 from infrahub.trigger.models import TriggerType
 from infrahub.trigger.setup import setup_triggers
 from infrahub.workflows.catalogue import (
@@ -173,15 +173,15 @@ async def update_computed_attribute_value_jinja2(
 
     await add_tags(branches=[branch_name], nodes=[obj.id], db_change=True)
 
-    macro_definition = MacroDefinition(macro=template_value)
-    my_filter = {}
-    for variable in macro_definition.variables:
+    jinja_template = Jinja2Template(template=template_value)
+    variables = {}
+    for variable in jinja_template.get_variables():
         components = variable.split("__")
         if len(components) == 2:
             property_name = components[0]
             property_value = components[1]
             attribute_property = getattr(obj, property_name)
-            my_filter[variable] = getattr(attribute_property, property_value)
+            variables[variable] = getattr(attribute_property, property_value)
         elif len(components) == 3:
             relationship_name = components[0]
             property_name = components[1]
@@ -189,11 +189,11 @@ async def update_computed_attribute_value_jinja2(
             relationship = getattr(obj, relationship_name)
             try:
                 attribute_property = getattr(relationship.peer, property_name)
-                my_filter[variable] = getattr(attribute_property, property_value)
+                variables[variable] = getattr(attribute_property, property_value)
             except ValueError:
-                my_filter[variable] = ""
+                variables[variable] = ""
 
-    value = macro_definition.render(variables=my_filter)
+    value = await jinja_template.render(variables=variables)
     existing_value = getattr(obj, attribute_name).value
     if value == existing_value:
         log.debug(f"Ignoring to update {obj} with existing value on {attribute_name}={value}")
@@ -235,7 +235,7 @@ async def process_jinja2(
         branch_name if branch_name in registry.get_altered_schema_branches() else registry.default_branch
     )
     schema_branch = registry.schema.get_schema_branch(name=target_branch_schema)
-    await service.client.schema.all(branch=branch_name, refresh=True)
+    await service.client.schema.all(branch=branch_name, refresh=True, schema_hash=schema_branch.get_hash())
 
     computed_macros = [
         attrib
@@ -311,35 +311,36 @@ async def trigger_update_jinja2_computed_attributes(
 async def computed_attribute_setup_jinja2(
     service: InfrahubServices, context: InfrahubContext, branch_name: str | None = None, event_name: str | None = None
 ) -> None:
-    log = get_run_logger()
+    async with service.database.start_session() as db:
+        log = get_run_logger()
 
-    if branch_name:
-        await add_tags(branches=[branch_name])
-        await wait_for_schema_to_converge(branch_name=branch_name, service=service, log=log)
+        if branch_name:
+            await add_tags(branches=[branch_name])
+            await wait_for_schema_to_converge(branch_name=branch_name, component=service.component, db=db, log=log)
 
-    triggers = await gather_trigger_computed_attribute_jinja2()
+        triggers = await gather_trigger_computed_attribute_jinja2()
 
-    for trigger in triggers:
-        if event_name != BranchDeletedEvent.event_name and trigger.branch == branch_name:
-            await service.workflow.submit_workflow(
-                workflow=TRIGGER_UPDATE_JINJA_COMPUTED_ATTRIBUTES,
-                context=context,
-                parameters={
-                    "branch_name": trigger.branch,
-                    "computed_attribute_name": trigger.computed_attribute.attribute.name,
-                    "computed_attribute_kind": trigger.computed_attribute.kind,
-                },
-            )
+        for trigger in triggers:
+            if event_name != BranchDeletedEvent.event_name and trigger.branch == branch_name:
+                await service.workflow.submit_workflow(
+                    workflow=TRIGGER_UPDATE_JINJA_COMPUTED_ATTRIBUTES,
+                    context=context,
+                    parameters={
+                        "branch_name": trigger.branch,
+                        "computed_attribute_name": trigger.computed_attribute.attribute.name,
+                        "computed_attribute_kind": trigger.computed_attribute.kind,
+                    },
+                )
 
-    # Configure all ComputedAttrJinja2Trigger in Prefect
-    async with get_client(sync_client=False) as prefect_client:
-        await setup_triggers(
-            client=prefect_client,
-            triggers=triggers,
-            trigger_type=TriggerType.COMPUTED_ATTR_JINJA2,
-        )  # type: ignore[misc]
+        # Configure all ComputedAttrJinja2Trigger in Prefect
+        async with get_client(sync_client=False) as prefect_client:
+            await setup_triggers(
+                client=prefect_client,
+                triggers=triggers,
+                trigger_type=TriggerType.COMPUTED_ATTR_JINJA2,
+            )  # type: ignore[misc]
 
-    log.info(f"{len(triggers)} Computed Attribute for Jinja2 automation configuration completed")
+        log.info(f"{len(triggers)} Computed Attribute for Jinja2 automation configuration completed")
 
 
 @flow(
@@ -353,45 +354,48 @@ async def computed_attribute_setup_python(
     event_name: str | None = None,
     commit: str | None = None,  # noqa: ARG001
 ) -> None:
-    log = get_run_logger()
+    async with service.database.start_session() as db:
+        log = get_run_logger()
 
-    branch_name = branch_name or registry.default_branch
+        branch_name = branch_name or registry.default_branch
 
-    if branch_name:
-        await add_tags(branches=[branch_name])
-        await wait_for_schema_to_converge(branch_name=branch_name, service=service, log=log)
+        if branch_name:
+            await add_tags(branches=[branch_name])
+            await wait_for_schema_to_converge(branch_name=branch_name, component=service.component, db=db, log=log)
 
-    triggers_python, triggers_python_query = await gather_trigger_computed_attribute_python(db=service.database)
+        triggers_python, triggers_python_query = await gather_trigger_computed_attribute_python(db=db)
 
-    for trigger in triggers_python:
-        if event_name != BranchDeletedEvent.event_name and trigger.branch == branch_name:
+        for trigger in triggers_python:
+            if event_name != BranchDeletedEvent.event_name and trigger.branch == branch_name:
+                log.info(
+                    f"Triggering update for {trigger.computed_attribute.computed_attribute.attribute.name} on {branch_name}"
+                )
+                await service.workflow.submit_workflow(
+                    workflow=TRIGGER_UPDATE_PYTHON_COMPUTED_ATTRIBUTES,
+                    context=context,
+                    parameters={
+                        "branch_name": branch_name,
+                        "computed_attribute_name": trigger.computed_attribute.computed_attribute.attribute.name,
+                        "computed_attribute_kind": trigger.computed_attribute.computed_attribute.kind,
+                    },
+                )
+
+        async with get_client(sync_client=False) as prefect_client:
+            await setup_triggers(
+                client=prefect_client,
+                triggers=triggers_python,
+                trigger_type=TriggerType.COMPUTED_ATTR_PYTHON,
+            )  # type: ignore[misc]
+            log.info(f"{len(triggers_python)} Computed Attribute for Python automation configuration completed")
+
+            await setup_triggers(
+                client=prefect_client,
+                triggers=triggers_python_query,
+                trigger_type=TriggerType.COMPUTED_ATTR_PYTHON_QUERY,
+            )  # type: ignore[misc]
             log.info(
-                f"Triggering update for {trigger.computed_attribute.computed_attribute.attribute.name} on {branch_name}"
+                f"{len(triggers_python_query)} Computed Attribute for Python Query automation configuration completed"
             )
-            await service.workflow.submit_workflow(
-                workflow=TRIGGER_UPDATE_PYTHON_COMPUTED_ATTRIBUTES,
-                context=context,
-                parameters={
-                    "branch_name": branch_name,
-                    "computed_attribute_name": trigger.computed_attribute.computed_attribute.attribute.name,
-                    "computed_attribute_kind": trigger.computed_attribute.computed_attribute.kind,
-                },
-            )
-
-    async with get_client(sync_client=False) as prefect_client:
-        await setup_triggers(
-            client=prefect_client,
-            triggers=triggers_python,
-            trigger_type=TriggerType.COMPUTED_ATTR_PYTHON,
-        )  # type: ignore[misc]
-        log.info(f"{len(triggers_python)} Computed Attribute for Python automation configuration completed")
-
-        await setup_triggers(
-            client=prefect_client,
-            triggers=triggers_python_query,
-            trigger_type=TriggerType.COMPUTED_ATTR_PYTHON_QUERY,
-        )  # type: ignore[misc]
-        log.info(f"{len(triggers_python_query)} Computed Attribute for Python Query automation configuration completed")
 
 
 @flow(
