@@ -31,7 +31,6 @@ class DiffMergeSerializer:
     def __init__(self, db: InfrahubDatabase, max_batch_size: int) -> None:
         self.db = db
         self.max_batch_size = max_batch_size
-        self._relationship_id_cache: dict[tuple[str, str], str] = {}
         self._attribute_type_cache: dict[tuple[str, str], type] = {}
         self._source_branch_name: str | None = None
         self._target_branch_name: str | None = None
@@ -39,7 +38,6 @@ class DiffMergeSerializer:
         self._conflicted_cardinality_one_relationships: set[tuple[str, str, str]] = set()
 
     def _reset_caches(self) -> None:
-        self._relationship_id_cache = {}
         self._attribute_type_cache = {}
 
     @property
@@ -70,20 +68,6 @@ class DiffMergeSerializer:
     def _to_action_str(self, action: DiffAction) -> str:
         return str(action.value).upper()
 
-    def _get_relationship_identifier(self, schema_kind: str, relationship_name: str) -> str:
-        cache_key = (schema_kind, relationship_name)
-        if cache_key in self._relationship_id_cache:
-            return self._relationship_id_cache[cache_key]
-        try:
-            node_schema = self._get_schema(kind=schema_kind, branch_name=self.source_branch_name)
-            relationship_schema = node_schema.get_relationship(name=relationship_name)
-        except (SchemaNotFoundError, ValueError):
-            node_schema = self._get_schema(kind=schema_kind, branch_name=self.target_branch_name)
-            relationship_schema = node_schema.get_relationship(name=relationship_name)
-        relationship_identifier = relationship_schema.get_identifier()
-        self._relationship_id_cache[cache_key] = relationship_identifier
-        return relationship_identifier
-
     def _get_property_type_for_attribute_value(self, schema_kind: str, attribute_name: str) -> type:
         cache_key = (schema_kind, attribute_name)
         if cache_key in self._attribute_type_cache:
@@ -113,8 +97,8 @@ class DiffMergeSerializer:
         ):
             return raw_value.lower() == "true"
         # this must be HAS_VALUE
-        if raw_value in (None, "NULL"):
-            return "NULL"
+        if raw_value in (None, NULL_VALUE):
+            return NULL_VALUE
         if value_type:
             if value_type is bool and isinstance(raw_value, str):
                 return raw_value.lower() == "true"
@@ -132,16 +116,13 @@ class DiffMergeSerializer:
                     for prop in element.properties:
                         if prop.property_type is not DatabaseEdgeType.IS_RELATED:
                             continue
-                        relationship_identifier = self._get_relationship_identifier(
-                            schema_kind=node.kind, relationship_name=rel.name
-                        )
                         if prop.previous_value:
                             self._conflicted_cardinality_one_relationships.add(
-                                (node.uuid, relationship_identifier, prop.previous_value)
+                                (node.uuid, rel.identifier, prop.previous_value)
                             )
                         if prop.new_value:
                             self._conflicted_cardinality_one_relationships.add(
-                                (node.uuid, relationship_identifier, prop.new_value)
+                                (node.uuid, rel.identifier, prop.new_value)
                             )
 
     async def serialize_diff(
@@ -181,13 +162,10 @@ class DiffMergeSerializer:
                 serialized_property_diffs.append(attribute_property_diff)
             relationship_diffs: list[RelationshipMergeDict] = []
             for rel_diff in node.relationships:
-                relationship_identifier = self._get_relationship_identifier(
-                    schema_kind=node.kind, relationship_name=rel_diff.name
-                )
                 for relationship_element_diff in rel_diff.relationships:
                     element_diffs, relationship_property_diffs = self._serialize_relationship_element(
                         relationship_diff=relationship_element_diff,
-                        relationship_identifier=relationship_identifier,
+                        relationship_identifier=rel_diff.identifier,
                         node_uuid=node.uuid,
                     )
                     relationship_diffs.extend(element_diffs)
@@ -251,8 +229,6 @@ class DiffMergeSerializer:
                 # we only delete attributes when the whole attribute is deleted
                 if action is DiffAction.REMOVED and attribute_diff.action is not DiffAction.REMOVED:
                     continue
-                    # action = DiffAction.ADDED
-                    # value = "NULL"
                 prop_dicts.append(
                     PropertyMergeDict(
                         property_type=property_diff.property_type.value,
@@ -318,10 +294,6 @@ class DiffMergeSerializer:
         actions_and_peers = self._get_actions_and_peers(relationship_diff=relationship_diff)
         added_peer_ids = [peer_id for action, peer_id in actions_and_peers if action is DiffAction.ADDED]
         removed_peer_ids = [peer_id for action, peer_id in actions_and_peers if action is DiffAction.REMOVED]
-        if not added_peer_ids:
-            added_peer_ids = [relationship_diff.peer_id]
-        if not removed_peer_ids:
-            removed_peer_ids = [relationship_diff.peer_id]
         for action, peer_id in actions_and_peers:
             if (
                 peer_id
@@ -338,6 +310,7 @@ class DiffMergeSerializer:
             relationship_diff_properties=relationship_diff.properties,
             added_peer_ids=added_peer_ids,
             removed_peer_ids=removed_peer_ids,
+            unchanged_peer_id=relationship_diff.peer_id,
         )
         return relationship_dicts, relationship_property_dicts
 
@@ -348,9 +321,14 @@ class DiffMergeSerializer:
         relationship_diff_properties: set[EnrichedDiffProperty],
         added_peer_ids: list[str],
         removed_peer_ids: list[str],
+        unchanged_peer_id: str,
     ) -> list[RelationshipPropertyMergeDict]:
-        added_property_dicts = self._get_default_property_merge_dicts(action=DiffAction.ADDED)
-        removed_property_dicts = self._get_default_property_merge_dicts(action=DiffAction.REMOVED)
+        added_property_dicts = {}
+        removed_property_dicts = {}
+        if added_peer_ids:
+            added_property_dicts = self._get_default_property_merge_dicts(action=DiffAction.ADDED)
+        if removed_peer_ids:
+            removed_property_dicts = self._get_default_property_merge_dicts(action=DiffAction.REMOVED)
         for property_diff in relationship_diff_properties:
             if property_diff.property_type is DatabaseEdgeType.IS_RELATED:
                 # handled above
@@ -372,10 +350,15 @@ class DiffMergeSerializer:
                 elif action is DiffAction.REMOVED:
                     removed_property_dicts[property_diff.property_type] = property_dict
         relationship_property_dicts = []
+        peers_and_property_dicts: list[tuple[str, dict[DatabaseEdgeType, PropertyMergeDict]]] = []
         if added_property_dicts:
-            peers_and_property_dicts = [(peer_id, added_property_dicts) for peer_id in added_peer_ids]
+            peers_and_property_dicts += [
+                (peer_id, added_property_dicts) for peer_id in (added_peer_ids or [unchanged_peer_id])
+            ]
         if removed_property_dicts:
-            peers_and_property_dicts += [(peer_id, removed_property_dicts) for peer_id in removed_peer_ids]
+            peers_and_property_dicts += [
+                (peer_id, removed_property_dicts) for peer_id in (removed_peer_ids or [unchanged_peer_id])
+            ]
         for peer_id, property_dicts in peers_and_property_dicts:
             if (
                 peer_id
