@@ -1,10 +1,17 @@
 from __future__ import annotations
 
+from collections import defaultdict
 from dataclasses import dataclass, field
-from typing import TYPE_CHECKING, Any, Optional
+from typing import TYPE_CHECKING, Any
 from uuid import uuid4
 
-from infrahub.core.constants import BranchSupportType, DiffAction, RelationshipCardinality, RelationshipStatus
+from infrahub.core.constants import (
+    NULL_VALUE,
+    BranchSupportType,
+    DiffAction,
+    RelationshipCardinality,
+    RelationshipStatus,
+)
 from infrahub.core.constants.database import DatabaseEdgeType
 from infrahub.core.timestamp import Timestamp
 
@@ -16,13 +23,11 @@ from .model.path import (
     DiffRelationship,
     DiffRoot,
     DiffSingleRelationship,
-    NodeFieldSpecifier,
 )
 
 if TYPE_CHECKING:
     from infrahub.core.branch import Branch
     from infrahub.core.query import QueryResult
-    from infrahub.core.schema import MainSchemaTypes
     from infrahub.core.schema.manager import SchemaManager
     from infrahub.core.schema.relationship_schema import RelationshipSchema
 
@@ -94,7 +99,7 @@ class DiffPropertyIntermediate:
         return self._ordered_values
 
     @property
-    def earliest_diff_value(self) -> Optional[DiffValueIntermediate]:
+    def earliest_diff_value(self) -> DiffValueIntermediate | None:
         ordered_values = self.get_ordered_values_asc()
         if not ordered_values:
             return None
@@ -119,22 +124,25 @@ class DiffPropertyIntermediate:
                 action = DiffAction.REMOVED
                 previous = lone_value.value
             return (action, lone_value.changed_at, previous, new)
-        previous_value = ordered_values[0].value
-        previous_status = ordered_values[0].status
+        earliest_value = ordered_values[0]
         new_diff_value = ordered_values[-1]
         new_value = new_diff_value.value
         new_status = new_diff_value.status
         action = DiffAction.UPDATED
-        if previous_value in (None, "NULL") and new_value not in (None, "NULL"):
+        previous_value = earliest_value.value
+        if earliest_value.changed_at > from_time:
             action = DiffAction.ADDED
-        elif (previous_value not in (None, "NULL") and new_value in (None, "NULL")) or (
-            previous_status,
+            previous_value = None
+        elif earliest_value.value in (None, NULL_VALUE) and new_value not in (None, NULL_VALUE):
+            action = DiffAction.ADDED
+        elif (earliest_value.value not in (None, NULL_VALUE) and new_value in (None, NULL_VALUE)) or (
+            earliest_value.status,
             new_status,
         ) == (RelationshipStatus.ACTIVE, RelationshipStatus.DELETED):
             action = DiffAction.REMOVED
-            if new_value != "NULL":
+            if new_value != NULL_VALUE:
                 new_value = None
-        elif previous_value == new_value or {previous_value, new_value} <= {None, "NULL"}:
+        elif earliest_value.value == new_value or {earliest_value.value, new_value} <= {None, NULL_VALUE}:
             action = DiffAction.UNCHANGED
         return (action, new_diff_value.changed_at, previous_value, new_value)
 
@@ -252,11 +260,11 @@ class DiffSingleRelationshipIntermediate:
         action = DiffAction.UPDATED
         if last_diff_prop.changed_at < from_time:
             action = DiffAction.UNCHANGED
-        elif previous_value in (None, "NULL") and new_value not in (None, "NULL"):
+        elif previous_value in (None, NULL_VALUE) and new_value not in (None, NULL_VALUE):
             action = DiffAction.ADDED
-        elif previous_value not in (None, "NULL") and new_value in (None, "NULL"):
+        elif previous_value not in (None, NULL_VALUE) and new_value in (None, NULL_VALUE):
             action = DiffAction.REMOVED
-        elif previous_value == new_value or {previous_value, new_value} <= {None, "NULL"}:
+        elif previous_value == new_value or {previous_value, new_value} <= {None, NULL_VALUE}:
             action = DiffAction.UNCHANGED
         return DiffProperty(
             property_type=property_type,
@@ -335,6 +343,7 @@ class DiffRelationshipIntermediate:
             )
         )
         to_time = database_path.property_to_time
+        # if the to time was set in the time frame, then it is effectively a delete
         if (
             to_time
             and database_path.property_from_time < diff_from_time <= to_time <= diff_to_time
@@ -387,8 +396,12 @@ class DiffNodeIntermediate(TrackedStatusUpdates):
     force_action: DiffAction | None
     uuid: str
     kind: str
+    db_id: str
+    from_time: Timestamp
+    status: RelationshipStatus
     attributes_by_name: dict[str, DiffAttributeIntermediate] = field(default_factory=dict)
-    relationships_by_name: dict[str, DiffRelationshipIntermediate] = field(default_factory=dict)
+    # {(name, identifier): DiffRelationshipIntermediate}
+    relationships_by_identifier: dict[tuple[str, str], DiffRelationshipIntermediate] = field(default_factory=dict)
 
     def to_diff_node(self, from_time: Timestamp, include_unchanged: bool) -> DiffNode:
         attributes = []
@@ -398,7 +411,7 @@ class DiffNodeIntermediate(TrackedStatusUpdates):
                 attributes.append(diff_attr)
         action, changed_at = self.get_action_and_timestamp(from_time=from_time)
         relationships = []
-        for rel in self.relationships_by_name.values():
+        for rel in self.relationships_by_identifier.values():
             diff_rel = rel.to_diff_relationship(include_unchanged=include_unchanged)
             if include_unchanged or diff_rel.action is not DiffAction.UNCHANGED:
                 relationships.append(diff_rel)
@@ -421,7 +434,7 @@ class DiffNodeIntermediate(TrackedStatusUpdates):
 
     @property
     def is_empty(self) -> bool:
-        return len(self.attributes_by_name) == 0 and len(self.relationships_by_name) == 0
+        return len(self.attributes_by_name) == 0 and len(self.relationships_by_identifier) == 0
 
 
 @dataclass
@@ -448,8 +461,8 @@ class DiffQueryParser:
         diff_branch: Branch,
         schema_manager: SchemaManager,
         from_time: Timestamp,
-        to_time: Optional[Timestamp] = None,
-        previous_node_field_specifiers: set[NodeFieldSpecifier] | None = None,
+        to_time: Timestamp | None = None,
+        previous_node_field_specifiers: dict[str, set[str]] | None = None,
     ) -> None:
         self.base_branch_name = base_branch.name
         self.diff_branch_name = diff_branch.name
@@ -463,7 +476,9 @@ class DiffQueryParser:
             self.diff_branched_from_time = Timestamp(diff_branch.get_branched_from())
         self._diff_root_by_branch: dict[str, DiffRootIntermediate] = {}
         self._final_diff_root_by_branch: dict[str, DiffRoot] = {}
-        self._previous_node_field_specifiers = previous_node_field_specifiers or set()
+        self._previous_node_field_specifiers = previous_node_field_specifiers or {}
+        self._new_node_field_specifiers: dict[str, set[str]] | None = None
+        self._current_node_field_specifiers: dict[str, set[str]] | None = None
 
     def get_branches(self) -> set[str]:
         return set(self._final_diff_root_by_branch.keys())
@@ -475,44 +490,70 @@ class DiffQueryParser:
             return self._final_diff_root_by_branch[branch]
         return DiffRoot(from_time=self.from_time, to_time=self.to_time, uuid=str(uuid4()), branch=branch, nodes=[])
 
-    def get_diff_node_field_specifiers(self) -> set[NodeFieldSpecifier]:
+    def get_diff_node_field_specifiers(self) -> dict[str, set[str]]:
         if self.diff_branch_name not in self._diff_root_by_branch:
-            return set()
-        node_field_specifiers: set[NodeFieldSpecifier] = set()
+            return {}
+        node_field_specifiers_map: dict[str, set[str]] = defaultdict(set)
         diff_root = self._diff_root_by_branch[self.diff_branch_name]
         for node in diff_root.nodes_by_id.values():
-            node_field_specifiers.update(
-                NodeFieldSpecifier(node_uuid=node.uuid, field_name=attribute_name)
-                for attribute_name in node.attributes_by_name
-            )
-            node_field_specifiers.update(
-                NodeFieldSpecifier(node_uuid=node.uuid, field_name=relationship_diff.identifier)
-                for relationship_diff in node.relationships_by_name.values()
-            )
-        return node_field_specifiers
+            for attribute_name in node.attributes_by_name:
+                node_field_specifiers_map[node.uuid].add(attribute_name)
+            for relationship_diff in node.relationships_by_identifier.values():
+                node_field_specifiers_map[node.uuid].add(relationship_diff.identifier)
+        return node_field_specifiers_map
 
-    def get_new_node_field_specifiers(self) -> set[NodeFieldSpecifier]:
+    def _remove_node_specifiers(
+        self, node_specifiers: dict[str, set[str]], node_specifiers_to_remove: dict[str, set[str]]
+    ) -> dict[str, set[str]]:
+        final_node_specifiers: dict[str, set[str]] = defaultdict(set)
+        for node_uuid, field_names_set in node_specifiers.items():
+            specifiers_to_remove = node_specifiers_to_remove.get(node_uuid, set())
+            final_specifiers = field_names_set - specifiers_to_remove
+            if final_specifiers:
+                final_node_specifiers[node_uuid] = final_specifiers
+        return final_node_specifiers
+
+    def get_new_node_field_specifiers(self) -> dict[str, set[str]]:
+        if self._new_node_field_specifiers is not None:
+            return self._new_node_field_specifiers
         branch_node_specifiers = self.get_diff_node_field_specifiers()
-        new_node_field_specifiers = branch_node_specifiers - self._previous_node_field_specifiers
+        new_node_field_specifiers = self._remove_node_specifiers(
+            branch_node_specifiers, self._previous_node_field_specifiers
+        )
+        self._new_node_field_specifiers = new_node_field_specifiers
         return new_node_field_specifiers
 
-    def get_current_node_field_specifiers(self) -> set[NodeFieldSpecifier]:
+    def get_current_node_field_specifiers(self) -> dict[str, set[str]]:
+        if self._current_node_field_specifiers is not None:
+            return self._current_node_field_specifiers
         new_node_field_specifiers = self.get_new_node_field_specifiers()
-        current_node_field_specifiers = self._previous_node_field_specifiers - new_node_field_specifiers
+        current_node_field_specifiers = self._remove_node_specifiers(
+            self._previous_node_field_specifiers, new_node_field_specifiers
+        )
+        self._current_node_field_specifiers = current_node_field_specifiers
         return current_node_field_specifiers
 
     def read_result(self, query_result: QueryResult) -> None:
         path = query_result.get_path(label="diff_path")
         database_path = DatabasePath.from_cypher_path(cypher_path=path)
         self._parse_path(database_path=database_path)
+        self._current_node_field_specifiers = None
+        self._new_node_field_specifiers = None
 
     def parse(self, include_unchanged: bool = False) -> None:
+        self._new_node_field_specifiers = None
+        self._current_node_field_specifiers = None
         if len(self._diff_root_by_branch) > 1:
             self._apply_base_branch_previous_values()
             self._remove_empty_base_diff_root()
         self._finalize(include_unchanged=include_unchanged)
 
     def _parse_path(self, database_path: DatabasePath) -> None:
+        to_time = database_path.property_to_time
+        # this path was added and removed within the timeframe, so we ignore it
+        if to_time and self.from_time <= database_path.property_from_time <= to_time <= self.to_time:
+            return
+
         diff_root = self._get_diff_root(database_path=database_path)
         diff_node = self._get_diff_node(database_path=database_path, diff_root=diff_root)
         self._update_attribute_level(database_path=database_path, diff_node=diff_node)
@@ -529,35 +570,53 @@ class DiffQueryParser:
             diff_root.nodes_by_id[node_id] = DiffNodeIntermediate(
                 uuid=node_id,
                 kind=database_path.node_kind,
+                db_id=database_path.node_db_id,
+                from_time=database_path.node_changed_at,
+                status=database_path.node_status,
                 force_action=DiffAction.UPDATED
                 if database_path.node_branch_support is BranchSupportType.AGNOSTIC
                 else None,
             )
         diff_node = diff_root.nodes_by_id[node_id]
+        # special handling for nodes that have their kind updated, which results in 2 nodes with the same uuid
+        if diff_node.db_id != database_path.node_db_id and (
+            database_path.node_changed_at > diff_node.from_time
+            or (
+                database_path.node_changed_at >= diff_node.from_time
+                and (diff_node.status, database_path.node_status)
+                == (RelationshipStatus.DELETED, RelationshipStatus.ACTIVE)
+            )
+        ):
+            diff_node.kind = database_path.node_kind
+            diff_node.db_id = database_path.node_db_id
+            diff_node.from_time = database_path.node_changed_at
+            diff_node.status = database_path.node_status
         diff_node.track_database_path(database_path=database_path)
         return diff_node
 
-    def _get_relationship_schema(
-        self, database_path: DatabasePath, node_schema: MainSchemaTypes
-    ) -> RelationshipSchema | None:
-        relationship_schemas = node_schema.get_relationships_by_identifier(id=database_path.attribute_name)
-        if len(relationship_schemas) == 1:
-            return relationship_schemas[0]
-        possible_path_directions = database_path.possible_relationship_directions
-        for rel_schema in relationship_schemas:
-            if rel_schema.direction in possible_path_directions:
-                return rel_schema
+    def _get_relationship_schema(self, database_path: DatabasePath) -> RelationshipSchema | None:
+        branches_to_check = [database_path.deepest_branch]
+        if database_path.deepest_branch == self.diff_branch_name:
+            branches_to_check.append(self.base_branch_name)
+        for schema_branch_name in branches_to_check:
+            node_schema = self.schema_manager.get(
+                name=database_path.node_kind, branch=schema_branch_name, duplicate=False
+            )
+            relationship_schemas = node_schema.get_relationships_by_identifier(id=database_path.attribute_name)
+            if len(relationship_schemas) == 1:
+                return relationship_schemas[0]
+            possible_path_directions = database_path.possible_relationship_directions
+            for rel_schema in relationship_schemas:
+                if rel_schema.direction in possible_path_directions:
+                    return rel_schema
         return None
 
     def _update_attribute_level(self, database_path: DatabasePath, diff_node: DiffNodeIntermediate) -> None:
-        node_schema = self.schema_manager.get(
-            name=database_path.node_kind, branch=database_path.deepest_branch, duplicate=False
-        )
         if "Attribute" in database_path.attribute_node.labels:
             diff_attribute = self._get_diff_attribute(database_path=database_path, diff_node=diff_node)
             self._update_attribute_property(database_path=database_path, diff_attribute=diff_attribute)
             return
-        relationship_schema = self._get_relationship_schema(database_path=database_path, node_schema=node_schema)
+        relationship_schema = self._get_relationship_schema(database_path=database_path)
         if not relationship_schema:
             return
         diff_relationship = self._get_diff_relationship(
@@ -571,11 +630,12 @@ class DiffQueryParser:
         self, database_path: DatabasePath, diff_node: DiffNodeIntermediate
     ) -> DiffAttributeIntermediate:
         attribute_name = database_path.attribute_name
-        node_field_specifier = NodeFieldSpecifier(node_uuid=diff_node.uuid, field_name=attribute_name)
         branch_name = database_path.deepest_branch
         from_time = self.from_time
-        if branch_name == self.base_branch_name and node_field_specifier in self.get_new_node_field_specifiers():
-            from_time = self.diff_branched_from_time
+        if branch_name == self.base_branch_name:
+            new_node_field_specifiers = self.get_new_node_field_specifiers()
+            if attribute_name in new_node_field_specifiers.get(diff_node.uuid, set()):
+                from_time = self.diff_branched_from_time
         if attribute_name not in diff_node.attributes_by_name:
             diff_node.attributes_by_name[attribute_name] = DiffAttributeIntermediate(
                 uuid=database_path.attribute_id,
@@ -610,22 +670,25 @@ class DiffQueryParser:
         relationship_schema: RelationshipSchema,
         database_path: DatabasePath,
     ) -> DiffRelationshipIntermediate:
-        diff_relationship = diff_node.relationships_by_name.get(relationship_schema.name)
+        diff_relationship = diff_node.relationships_by_identifier.get(
+            (relationship_schema.name, relationship_schema.get_identifier())
+        )
         if not diff_relationship:
-            node_field_specifier = NodeFieldSpecifier(
-                node_uuid=diff_node.uuid, field_name=relationship_schema.get_identifier()
-            )
             branch_name = database_path.deepest_branch
             from_time = self.from_time
-            if branch_name == self.base_branch_name and node_field_specifier in self.get_new_node_field_specifiers():
-                from_time = self.diff_branched_from_time
+            if branch_name == self.base_branch_name:
+                new_node_field_specifiers = self.get_new_node_field_specifiers()
+                if relationship_schema.get_identifier() in new_node_field_specifiers.get(diff_node.uuid, set()):
+                    from_time = self.diff_branched_from_time
             diff_relationship = DiffRelationshipIntermediate(
                 name=relationship_schema.name,
                 cardinality=relationship_schema.cardinality,
                 identifier=relationship_schema.get_identifier(),
                 from_time=from_time,
             )
-            diff_node.relationships_by_name[relationship_schema.name] = diff_relationship
+            diff_node.relationships_by_identifier[relationship_schema.name, relationship_schema.get_identifier()] = (
+                diff_relationship
+            )
         return diff_relationship
 
     def _apply_base_branch_previous_values(self) -> None:
@@ -662,15 +725,15 @@ class DiffQueryParser:
     def _apply_relationship_previous_values(
         self, diff_node: DiffNodeIntermediate, base_diff_node: DiffNodeIntermediate
     ) -> None:
-        for relationship_name, diff_relationship in diff_node.relationships_by_name.items():
-            base_diff_relationship = base_diff_node.relationships_by_name.get(relationship_name)
+        for relationship_key, diff_relationship in diff_node.relationships_by_identifier.items():
+            base_diff_relationship = base_diff_node.relationships_by_identifier.get(relationship_key)
             if not base_diff_relationship:
                 continue
             for db_id, property_set in diff_relationship.properties_by_db_id.items():
                 base_property_set = base_diff_relationship.properties_by_db_id.get(db_id)
                 if not base_property_set:
                     continue
-                base_diff_property_by_type: dict[DatabaseEdgeType, Optional[DiffRelationshipPropertyIntermediate]] = {
+                base_diff_property_by_type: dict[DatabaseEdgeType, DiffRelationshipPropertyIntermediate | None] = {
                     DatabaseEdgeType(p.property_type): None for p in property_set
                 }
                 base_diff_property_by_type[DatabaseEdgeType.IS_RELATED] = None
@@ -716,7 +779,7 @@ class DiffQueryParser:
                         continue
                     if ordered_diff_values[-1].changed_at >= self.diff_branched_from_time:
                         return
-            for relationship_diff in node_diff.relationships_by_name.values():
+            for relationship_diff in node_diff.relationships_by_identifier.values():
                 for diff_relationship_property_list in relationship_diff.properties_by_db_id.values():
                     for diff_relationship_property in diff_relationship_property_list:
                         if diff_relationship_property.changed_at >= self.diff_branched_from_time:
