@@ -6,6 +6,8 @@ from collections import defaultdict
 from itertools import chain, combinations
 from typing import Any
 
+from infrahub_sdk.template import Jinja2Template
+from infrahub_sdk.template.exceptions import JinjaTemplateError, JinjaTemplateOperationViolationError
 from infrahub_sdk.topological_sort import DependencyCycleExistsError, topological_sort
 from infrahub_sdk.utils import compare_lists, deep_merge_dict, duplicates, intersection
 from typing_extensions import Self
@@ -51,11 +53,11 @@ from infrahub.core.schema.definitions.core import core_profile_schema_definition
 from infrahub.core.validators import CONSTRAINT_VALIDATOR_MAP
 from infrahub.exceptions import SchemaNotFoundError, ValidationError
 from infrahub.log import get_logger
-from infrahub.support.macro import MacroDefinition
 from infrahub.types import ATTRIBUTE_TYPES
 from infrahub.utils import format_label
 from infrahub.visuals import select_color
 
+from ... import config
 from ..constants.schema import PARENT_CHILD_IDENTIFIER
 from .constants import INTERNAL_SCHEMA_NODE_KINDS, SchemaNamespace
 from .schema_branch_computed import ComputedAttributes
@@ -815,11 +817,16 @@ class SchemaBranch:
                         f"{node_schema.namespace}{node_schema.name}: default value {exc.message}"
                     ) from exc
 
-    def _is_attr_combination_unique(self, attrs_paths: list[str], uniqueness_constraints: list[list[str]]) -> bool:
+    def _is_attr_combination_unique(
+        self, attrs_paths: list[str], uniqueness_constraints: list[list[str]] | None
+    ) -> bool:
         """
         Return whether at least one combination of any length of `attrs_paths` is equal to a uniqueness constraint.
         """
 
+        if not uniqueness_constraints:
+            return False
+
         unique_constraint_group_sets = [set(ucg) for ucg in uniqueness_constraints]
         for i in range(1, len(attrs_paths) + 1):
             for attr_combo in combinations(attrs_paths, i):
@@ -854,12 +861,13 @@ class SchemaBranch:
                         rel_schemas_to_paths[rel_identifier] = (schema_path.related_schema, [])
                     rel_schemas_to_paths[rel_identifier][1].append(schema_path.attribute_path_as_str)
 
-            # For every relationship referred within hfid, check whether the combination of attributes is unique is the peer schema node
-            for related_schema, attrs_paths in rel_schemas_to_paths.values():
-                if not self._is_attr_combination_unique(attrs_paths, related_schema.uniqueness_constraints):
-                    raise ValidationError(
-                        f"HFID of {node_schema.kind} refers peer {related_schema.kind} with a non-unique combination of attributes {attrs_paths}"
-                    )
+            if config.SETTINGS.main.schema_strict_mode:
+                # For every relationship referred within hfid, check whether the combination of attributes is unique is the peer schema node
+                for related_schema, attrs_paths in rel_schemas_to_paths.values():
+                    if not self._is_attr_combination_unique(attrs_paths, related_schema.uniqueness_constraints):
+                        raise ValidationError(
+                            f"HFID of {node_schema.kind} refers peer {related_schema.kind} with a non-unique combination of attributes {attrs_paths}"
+                        )
 
     def validate_required_relationships(self) -> None:
         reverse_dependency_map: dict[str, set[str]] = {}
@@ -997,9 +1005,11 @@ class SchemaBranch:
             generic_schema = self.get_generic(name=name, duplicate=False)
             for attribute in generic_schema.attributes:
                 if attribute.computed_attribute and attribute.computed_attribute.kind != ComputedAttributeKind.USER:
-                    raise ValueError(
-                        f"{generic_schema.kind}: Attribute {attribute.name!r} computed attributes are only allowed on nodes not generics"
-                    )
+                    for inheriting_node in generic_schema.used_by:
+                        node_schema = self.get_node(name=inheriting_node, duplicate=False)
+                        self.computed_attributes.validate_generic_inheritance(
+                            node=node_schema, attribute=attribute, generic=generic_schema
+                        )
 
     def _validate_computed_attribute(self, node: NodeSchema, attribute: AttributeSchema) -> None:
         if not attribute.computed_attribute or attribute.computed_attribute.kind == ComputedAttributeKind.USER:
@@ -1030,14 +1040,22 @@ class SchemaBranch:
                 | SchemaElementPathType.REL_ONE_MANDATORY_ATTR_WITH_PROP
                 | SchemaElementPathType.REL_ONE_ATTR_WITH_PROP
             )
+
+            jinja_template = Jinja2Template(template=attribute.computed_attribute.jinja2_template)
             try:
-                macro = MacroDefinition(macro=attribute.computed_attribute.jinja2_template)
-            except ValueError as exc:
+                variables = jinja_template.get_variables()
+                jinja_template.validate(restricted=config.SETTINGS.security.restrict_untrusted_jinja2_filters)
+            except JinjaTemplateOperationViolationError as exc:
                 raise ValueError(
-                    f"{node.kind}: Attribute {attribute.name!r} is assigned by a jinja2 template, but has an invalid template"
+                    f"{node.kind}: Attribute {attribute.name!r} is assigned by a jinja2 template, but has an invalid template: {exc.message}"
                 ) from exc
 
-            for variable in macro.variables:
+            except JinjaTemplateError as exc:
+                raise ValueError(
+                    f"{node.kind}: Attribute {attribute.name!r} is assigned by a jinja2 template, but has an invalid template: : {exc.message}"
+                ) from exc
+
+            for variable in variables:
                 try:
                     schema_path = self.validate_schema_path(
                         node_schema=node, path=variable, allowed_path_types=allowed_path_types
@@ -1167,25 +1185,25 @@ class SchemaBranch:
                 self.set(name=schema_to_update.kind, schema=schema_to_update)
 
     def process_human_friendly_id(self) -> None:
+        """
+        For each schema node, if there is no HFID defined, set it with:
+        - The first unique attribute if existing
+        - Otherwise the first uniqueness constraint with a single attribute
+
+        Also, HFID is added to the uniqueness constraints.
+        """
         for name in self.generic_names_without_templates + self.node_names:
             node = self.get(name=name, duplicate=False)
 
-            # If human_friendly_id IS NOT defined
-            #   but some the model has some unique attribute, we generate a human_friendly_id
-            # If human_friendly_id IS defined
-            #   but no unique attributes and no uniquess constraints, we add a uniqueness_constraint
             if not node.human_friendly_id:
                 if node.unique_attributes:
-                    for attr in node.unique_attributes:
-                        node = self.get(name=name, duplicate=True)
-                        node.human_friendly_id = [f"{attr.name}__value"]
-                        self.set(name=node.kind, schema=node)
-                        break
-                    continue
+                    node = self.get(name=name, duplicate=True)
+                    node.human_friendly_id = [f"{node.unique_attributes[0].name}__value"]
+                    self.set(name=node.kind, schema=node)
 
                 # if no human_friendly_id and a uniqueness_constraint with a single attribute exists
                 # then use that attribute as the human_friendly_id
-                if node.uniqueness_constraints:
+                elif node.uniqueness_constraints:
                     for constraint_paths in node.uniqueness_constraints:
                         if len(constraint_paths) > 1:
                             continue
@@ -1202,22 +1220,15 @@ class SchemaBranch:
                             break
 
             # Add hfid to uniqueness constraint
-            if node.human_friendly_id:
-                uniqueness_constraint: list[str] = []
-                for item in node.human_friendly_id:
-                    schema_attribute_path = node.parse_schema_path(path=item, schema=self)
-                    if schema_attribute_path.is_type_attribute:
-                        uniqueness_constraint.append(item)
-                    elif schema_attribute_path.is_type_relationship:
-                        uniqueness_constraint.append(schema_attribute_path.relationship_schema.name)
-
+            hfid_uniqueness_constraint = node.convert_hfid_to_uniqueness_constraint(schema_branch=self)
+            if hfid_uniqueness_constraint:
                 node = self.get(name=name, duplicate=True)
                 # Make sure there is no duplicate regarding generics values.
                 if node.uniqueness_constraints:
-                    if uniqueness_constraint not in node.uniqueness_constraints:
-                        node.uniqueness_constraints.append(uniqueness_constraint)
+                    if hfid_uniqueness_constraint not in node.uniqueness_constraints:
+                        node.uniqueness_constraints.append(hfid_uniqueness_constraint)
                 else:
-                    node.uniqueness_constraints = [uniqueness_constraint]
+                    node.uniqueness_constraints = [hfid_uniqueness_constraint]
                 self.set(name=node.kind, schema=node)
 
     def process_hierarchy(self) -> None:
@@ -1237,7 +1248,7 @@ class SchemaBranch:
             node = node.duplicate()
             changed = False
 
-            if node.hierarchy not in node.inherit_from:
+            if node.hierarchy and node.hierarchy not in node.inherit_from:
                 node.inherit_from.append(node.hierarchy)
                 changed = True
 
@@ -1256,6 +1267,23 @@ class SchemaBranch:
             if changed:
                 self.set(name=name, schema=node)
 
+    def _get_generic_fields_map(
+        self, node_schema: MainSchemaTypes
+    ) -> dict[str, tuple[GenericSchema, AttributeSchema | RelationshipSchema]]:
+        generic_fields_map: dict[str, tuple[GenericSchema, AttributeSchema | RelationshipSchema]] = {}
+        if isinstance(node_schema, NodeSchema) and node_schema.inherit_from:
+            for generic_kind in node_schema.inherit_from:
+                generic_schema = self.get_generic(name=generic_kind, duplicate=False)
+                for generic_attr in generic_schema.attributes:
+                    if generic_attr.name in node_schema.attribute_names:
+                        generic_fields_map[generic_attr.name] = (generic_schema, generic_attr)
+                        continue
+                for generic_rel in generic_schema.relationships:
+                    if generic_rel.name in node_schema.relationship_names:
+                        generic_fields_map[generic_rel.name] = (generic_schema, generic_rel)
+                        continue
+        return generic_fields_map
+
     def process_inheritance(self) -> None:
         """Extend all the nodes with the attributes and relationships
         from the Interface objects defined in inherited_from.
@@ -1308,7 +1336,7 @@ class SchemaBranch:
 
         # Update all generics with the list of nodes referrencing them.
         for generic_name in self.generics.keys():
-            generic = self.get(name=generic_name)
+            generic = self.get_generic(name=generic_name)
 
             if generic.kind in generics_used_by:
                 generic.used_by = sorted(generics_used_by[generic.kind])
@@ -1326,40 +1354,46 @@ class SchemaBranch:
         for name in self.all_names:
             node = self.get(name=name, duplicate=False)
 
-            # Check if this node requires a change before duplicating
-            change_required = False
-            for attr in node.attributes:
-                if attr.branch is None:
-                    change_required = True
-                    break
-            if not change_required:
-                for rel in node.relationships:
-                    if rel.branch is None:
-                        change_required = True
-                        break
+            generic_fields_map = self._get_generic_fields_map(node_schema=node)
 
-            if not change_required:
+            attrs_to_update: dict[str, BranchSupportType] = {}
+            for attr in node.attributes:
+                if attr.inherited and attr.name in generic_fields_map:
+                    generic_schema, generic_attr = generic_fields_map[attr.name]
+                    if attr.branch == generic_schema.branch == generic_attr.branch != node.branch:
+                        attrs_to_update[attr.name] = node.branch
+
+                if attr.branch is None:
+                    attrs_to_update[attr.name] = node.branch
+
+            rels_to_update: dict[str, BranchSupportType] = {}
+            for rel in node.relationships:
+                if not rel.inherited and rel.branch is not None:
+                    continue
+                needs_update = rel.branch is None
+                if needs_update is False and rel.inherited and rel.name in generic_fields_map:
+                    generic_schema, generic_rel = generic_fields_map[rel.name]
+                    if rel.branch == generic_schema.branch == generic_rel.branch != node.branch:
+                        needs_update = True
+                if needs_update:
+                    peer_node = self.get(name=rel.peer, duplicate=False)
+                    if node.branch == peer_node.branch:
+                        rels_to_update[rel.name] = node.branch
+                    elif BranchSupportType.LOCAL in (node.branch, peer_node.branch):
+                        rels_to_update[rel.name] = BranchSupportType.LOCAL
+                    else:
+                        rels_to_update[rel.name] = BranchSupportType.AWARE
+
+            if not attrs_to_update and not rels_to_update:
                 continue
 
             node = node.duplicate()
-
-            for attr in node.attributes:
-                if attr.branch is not None:
-                    continue
-
-                attr.branch = node.branch
-
-            for rel in node.relationships:
-                if rel.branch is not None:
-                    continue
-
-                peer_node = self.get(name=rel.peer, duplicate=False)
-                if node.branch == peer_node.branch:
-                    rel.branch = node.branch
-                elif BranchSupportType.LOCAL in (node.branch, peer_node.branch):
-                    rel.branch = BranchSupportType.LOCAL
-                else:
-                    rel.branch = BranchSupportType.AWARE
+            for node_attr in node.attributes:
+                if node_attr.name in attrs_to_update:
+                    node_attr.branch = attrs_to_update[node_attr.name]
+            for node_rel in node.relationships:
+                if node_rel.name in rels_to_update:
+                    node_rel.branch = rels_to_update[node_rel.name]
 
             self.set(name=name, schema=node)
 
@@ -1452,14 +1486,15 @@ class SchemaBranch:
             self.set(name=name, schema=node)
 
     def generate_weight(self) -> None:
-        for name in self.all_names:
+        for name in self.generic_names:
             node = self.get(name=name, duplicate=False)
+
             items_to_update = [item for item in node.attributes + node.relationships if not item.order_weight]
+
             if not items_to_update:
                 continue
 
             node = node.duplicate()
-
             current_weight = 0
             for item in node.attributes + node.relationships:
                 current_weight += 1000
@@ -1468,6 +1503,30 @@ class SchemaBranch:
 
             self.set(name=name, schema=node)
 
+        for name in self.node_names + self.profile_names:
+            node = self.get(name=name, duplicate=False)
+
+            items_to_update = [item for item in node.attributes + node.relationships if not item.order_weight]
+
+            if not items_to_update:
+                continue
+            node = node.duplicate()
+
+            generic_fields_map = self._get_generic_fields_map(node_schema=node)
+
+            current_weight = 0
+            for item in node.attributes + node.relationships:
+                current_weight += 1000
+                if not item.order_weight:
+                    if item.inherited:
+                        _, generic_field = generic_fields_map[item.name]
+                        if generic_field:
+                            item.order_weight = generic_field.order_weight
+                    if not item.order_weight:
+                        item.order_weight = current_weight
+
+            self.set(name=name, schema=node)
+
     def cleanup_inherited_elements(self) -> None:
         for name in self.node_names:
             node = self.get_node(name=name, duplicate=False)
@@ -1850,10 +1909,8 @@ class SchemaBranch:
 
                 self.set(name=node_name, schema=node_schema)
 
-    def add_relationships_to_template(self, node: NodeSchema) -> None:
+    def add_relationships_to_template(self, node: NodeSchema | GenericSchema) -> None:
         template_schema = self.get(name=self._get_object_template_kind(node_kind=node.kind), duplicate=False)
-        if template_schema.is_generic_schema:
-            return
 
         # Remove previous relationships to account for new ones
         template_schema.relationships = [
@@ -1895,13 +1952,13 @@ class SchemaBranch:
                     label=f"{relationship.name} template".title()
                     if relationship.kind in [RelationshipKind.COMPONENT, RelationshipKind.PARENT]
                     else relationship.name.title(),
+                    inherited=relationship.inherited,
                 )
             )
 
-            if relationship.kind == RelationshipKind.PARENT:
-                template_schema.human_friendly_id = [
-                    f"{relationship.name}__template_name__value"
-                ] + template_schema.human_friendly_id
+            parent_hfid = f"{relationship.name}__template_name__value"
+            if relationship.kind == RelationshipKind.PARENT and parent_hfid not in template_schema.human_friendly_id:
+                template_schema.human_friendly_id = [parent_hfid] + template_schema.human_friendly_id
                 template_schema.uniqueness_constraints[0].append(relationship.name)
 
     def generate_object_template_from_node(
@@ -1925,9 +1982,6 @@ class SchemaBranch:
         need_template_kinds = [n.kind for n in need_templates]
 
         if node.is_generic_schema:
-            # When needing a template for a generic, we generate an empty shell mostly to make sure that schemas (including the GraphQL one) will
-            # look right. We don't really care about applying inheritance of fields as it was already processed and actual templates will have the
-            # correct attributes and relationships
             template = GenericSchema(
                 name=node.kind,
                 namespace="Template",
@@ -1936,43 +1990,44 @@ class SchemaBranch:
                 generate_profile=False,
                 branch=node.branch,
                 include_in_menu=False,
+                display_labels=["template_name__value"],
+                human_friendly_id=["template_name__value"],
+                uniqueness_constraints=[["template_name__value"]],
                 attributes=[template_name_attr],
             )
 
             for used in node.used_by:
                 if used in need_template_kinds:
                     template.used_by.append(self._get_object_template_kind(node_kind=used))
+        else:
+            template = TemplateSchema(
+                name=node.kind,
+                namespace="Template",
+                label=f"Object template {node.label}",
+                description=f"Object template for {node.kind}",
+                branch=node.branch,
+                include_in_menu=False,
+                display_labels=["template_name__value"],
+                human_friendly_id=["template_name__value"],
+                uniqueness_constraints=[["template_name__value"]],
+                inherit_from=[InfrahubKind.LINEAGESOURCE, InfrahubKind.NODE, core_template_schema.kind],
+                default_filter="template_name__value",
+                attributes=[template_name_attr],
+                relationships=[
+                    RelationshipSchema(
+                        name="related_nodes",
+                        identifier="node__objecttemplate",
+                        peer=node.kind,
+                        kind=RelationshipKind.TEMPLATE,
+                        cardinality=RelationshipCardinality.MANY,
+                        branch=BranchSupportType.AWARE,
+                    )
+                ],
+            )
 
-            return template
-
-        template = TemplateSchema(
-            name=node.kind,
-            namespace="Template",
-            label=f"Object template {node.label}",
-            description=f"Object template for {node.kind}",
-            branch=node.branch,
-            include_in_menu=False,
-            display_labels=["template_name__value"],
-            human_friendly_id=["template_name__value"],
-            uniqueness_constraints=[["template_name__value"]],
-            inherit_from=[InfrahubKind.LINEAGESOURCE, InfrahubKind.NODE, core_template_schema.kind],
-            default_filter="template_name__value",
-            attributes=[template_name_attr],
-            relationships=[
-                RelationshipSchema(
-                    name="related_nodes",
-                    identifier="node__objecttemplate",
-                    peer=node.kind,
-                    kind=RelationshipKind.TEMPLATE,
-                    cardinality=RelationshipCardinality.MANY,
-                    branch=BranchSupportType.AWARE,
-                )
-            ],
-        )
-
-        for inherited in node.inherit_from:
-            if inherited in need_template_kinds:
-                template.inherit_from.append(self._get_object_template_kind(node_kind=inherited))
+            for inherited in node.inherit_from:
+                if inherited in need_template_kinds:
+                    template.inherit_from.append(self._get_object_template_kind(node_kind=inherited))
 
         for node_attr in node.attributes:
             if node_attr.unique or node_attr.read_only:
@@ -1980,7 +2035,7 @@ class SchemaBranch:
 
             attr = AttributeSchema(
                 optional=node_attr.optional if is_autogenerated_subtemplate else True,
-                **node_attr.model_dump(exclude=["id", "unique", "optional", "read_only", "inherited"]),
+                **node_attr.model_dump(exclude=["id", "unique", "optional", "read_only"]),
             )
             template.attributes.append(attr)
 
