@@ -4,6 +4,7 @@ from typing import TYPE_CHECKING, Any
 
 from graphql.type.definition import GraphQLNonNull
 from infrahub_sdk.utils import extract_fields
+from opentelemetry import trace
 
 from infrahub.core.constants import BranchSupportType, InfrahubKind, RelationshipHierarchyDirection
 from infrahub.core.manager import NodeManager
@@ -20,6 +21,7 @@ if TYPE_CHECKING:
     from infrahub.graphql.initialization import GraphqlContext
 
 
+@trace.get_tracer(__name__).start_as_current_span("account_resolver")
 async def account_resolver(
     root: dict,  # noqa: ARG001
     info: GraphQLResolveInfo,
@@ -44,6 +46,7 @@ async def account_resolver(
         )
 
 
+@trace.get_tracer(__name__).start_as_current_span("default_resolver")
 async def default_resolver(*args: Any, **kwargs) -> dict | list[dict] | None:
     """Not sure why but the default resolver returns sometime 4 positional args and sometime 2.
 
@@ -126,6 +129,7 @@ async def default_resolver(*args: Any, **kwargs) -> dict | list[dict] | None:
         return await objs[0].to_graphql(db=db, fields=fields, related_node_ids=graphql_context.related_node_ids)
 
 
+@trace.get_tracer(__name__).start_as_current_span("parent_field_name_resolver")
 async def parent_field_name_resolver(parent: dict[str, dict], info: GraphQLResolveInfo) -> dict:
     """This resolver gets used when we know that the parent resolver has already gathered the required information.
 
@@ -135,6 +139,7 @@ async def parent_field_name_resolver(parent: dict[str, dict], info: GraphQLResol
     return parent[info.field_name]
 
 
+@trace.get_tracer(__name__).start_as_current_span("default_paginated_list_resolver")
 async def default_paginated_list_resolver(
     root: dict,  # noqa: ARG001
     info: GraphQLResolveInfo,
@@ -225,12 +230,14 @@ async def default_paginated_list_resolver(
         return response
 
 
+@trace.get_tracer(__name__).start_as_current_span("single_relationship_resolver")
 async def single_relationship_resolver(parent: dict, info: GraphQLResolveInfo, **kwargs: Any) -> dict[str, Any]:
     graphql_context: GraphqlContext = info.context
     resolver = graphql_context.single_relationship_resolver
     return await resolver.resolve(parent=parent, info=info, **kwargs)
 
 
+@trace.get_tracer(__name__).start_as_current_span("many_relationship_resolver")
 async def many_relationship_resolver(
     parent: dict, info: GraphQLResolveInfo, include_descendants: bool | None = False, **kwargs: Any
 ) -> dict[str, Any]:
@@ -251,6 +258,7 @@ async def descendants_resolver(parent: dict, info: GraphQLResolveInfo, **kwargs)
     )
 
 
+@trace.get_tracer(__name__).start_as_current_span("hierarchy_resolver")
 async def hierarchy_resolver(
     direction: RelationshipHierarchyDirection, parent: dict, info: GraphQLResolveInfo, **kwargs
 ) -> dict[str, Any]:
