@@ -40,6 +40,7 @@ class InfrahubGenerator:
         self.generator_instance = generator_instance
         self._init_client = client.clone()
         self._init_client.config.default_branch = self._init_client.default_branch = self.branch_name
+        self._init_client.store._default_branch = self.branch_name
         self._client: InfrahubClient | None = None
         self._nodes: list[InfrahubNode] = []
         self._related_nodes: list[InfrahubNode] = []
@@ -125,7 +126,7 @@ class InfrahubGenerator:
         await self._init_client.schema.all(branch=self.branch_name)
 
         for kind in data:
-            if kind in self._init_client.schema.cache[self.branch_name]:
+            if kind in self._init_client.schema.cache[self.branch_name].nodes.keys():
                 for result in data[kind].get("edges", []):
                     node = await self.infrahub_node.from_graphql(
                         client=self._init_client, branch=self.branch_name, data=result
@@ -137,7 +138,7 @@ class InfrahubGenerator:
 
         for node in self._nodes + self._related_nodes:
             if node.id:
-                self._init_client.store.set(key=node.id, node=node)
+                self._init_client.store.set(node=node)
 
     @abstractmethod
     async def generate(self, data: dict) -> None:
