@@ -15,7 +15,7 @@ from .exceptions import (
 )
 from .graphql import Mutation, Query
 from .schema import GenericSchemaAPI, RelationshipCardinality, RelationshipKind
-from .utils import compare_lists, get_flat_value
+from .utils import compare_lists, generate_short_id, get_flat_value
 from .uuidt import UUIDT
 
 if TYPE_CHECKING:
@@ -43,6 +43,20 @@ ARTIFACT_DEFINITION_GENERATE_FEATURE_NOT_SUPPORTED_MESSAGE = (
     "calling generate is only supported for CoreArtifactDefinition nodes"
 )
 
+HFID_STR_SEPARATOR = "__"
+
+
+def parse_human_friendly_id(hfid: str | list[str]) -> tuple[str | None, list[str]]:
+    """Parse a human friendly ID into a kind and an identifier."""
+    if isinstance(hfid, str):
+        hfid_parts = hfid.split(HFID_STR_SEPARATOR)
+        if len(hfid_parts) == 1:
+            return None, hfid_parts
+        return hfid_parts[0], hfid_parts[1:]
+    if isinstance(hfid, list):
+        return None, hfid
+    raise ValueError(f"Invalid human friendly ID: {hfid}")
+
 
 class Attribute:
     """Represents an attribute of a Node, including its schema, value, and properties."""
@@ -68,17 +82,18 @@ class Attribute:
 
         self.id: str | None = data.get("id", None)
 
-        self.value: Any | None = data.get("value", None)
+        self._value: Any | None = data.get("value", None)
+        self.value_has_been_mutated = False
         self.is_default: bool | None = data.get("is_default", None)
         self.is_from_profile: bool | None = data.get("is_from_profile", None)
 
-        if self.value:
+        if self._value:
             value_mapper: dict[str, Callable] = {
                 "IPHost": ipaddress.ip_interface,
                 "IPNetwork": ipaddress.ip_network,
             }
             mapper = value_mapper.get(schema.kind, lambda value: value)
-            self.value = mapper(data.get("value"))
+            self._value = mapper(data.get("value"))
 
         self.is_inherited: bool | None = data.get("is_inherited", None)
         self.updated_at: str | None = data.get("updated_at", None)
@@ -93,6 +108,15 @@ class Attribute:
             if data.get(prop_name):
                 setattr(self, prop_name, NodeProperty(data=data.get(prop_name)))  # type: ignore[arg-type]
 
+    @property
+    def value(self) -> Any:
+        return self._value
+
+    @value.setter
+    def value(self, value: Any) -> None:
+        self._value = value
+        self.value_has_been_mutated = True
+
     def _generate_input_data(self) -> dict | None:
         data: dict[str, Any] = {}
         variables: dict[str, Any] = {}
@@ -340,10 +364,10 @@ class RelatedNode(RelatedNodeBase):
             return self._peer  # type: ignore[return-value]
 
         if self.id and self.typename:
-            return self._client.store.get(key=self.id, kind=self.typename)  # type: ignore[return-value]
+            return self._client.store.get(key=self.id, kind=self.typename, branch=self._branch)  # type: ignore[return-value]
 
         if self.hfid_str:
-            return self._client.store.get_by_hfid(key=self.hfid_str)  # type: ignore[return-value]
+            return self._client.store.get(key=self.hfid_str, branch=self._branch)  # type: ignore[return-value]
 
         raise ValueError("Node must have at least one identifier (ID or HFID) to query it.")
 
@@ -387,10 +411,10 @@ class RelatedNodeSync(RelatedNodeBase):
             return self._peer  # type: ignore[return-value]
 
         if self.id and self.typename:
-            return self._client.store.get(key=self.id, kind=self.typename)  # type: ignore[return-value]
+            return self._client.store.get(key=self.id, kind=self.typename, branch=self._branch)  # type: ignore[return-value]
 
         if self.hfid_str:
-            return self._client.store.get_by_hfid(key=self.hfid_str)  # type: ignore[return-value]
+            return self._client.store.get(key=self.hfid_str, branch=self._branch)  # type: ignore[return-value]
 
         raise ValueError("Node must have at least one identifier (ID or HFID) to query it.")
 
@@ -678,6 +702,11 @@ class InfrahubNodeBase:
         self._branch = branch
         self._existing: bool = True
 
+        # Generate a unique ID only to be used inside the SDK
+        # The format if this ID is purposely different from the ID used by the API
+        # This is done to avoid confusion and potential conflicts between the IDs
+        self._internal_id = generate_short_id()
+
         self.id = data.get("id", None) if isinstance(data, dict) else None
         self.display_label: str | None = data.get("display_label", None) if isinstance(data, dict) else None
         self.typename: str | None = data.get("__typename", schema.kind) if isinstance(data, dict) else schema.kind
@@ -694,6 +723,9 @@ class InfrahubNodeBase:
         self._init_attributes(data)
         self._init_relationships(data)
 
+    def get_branch(self) -> str:
+        return self._branch
+
     def get_path_value(self, path: str) -> Any:
         path_parts = path.split("__")
         return_value = None
@@ -794,6 +826,11 @@ class InfrahubNodeBase:
     def get_kind(self) -> str:
         return self._schema.kind
 
+    def get_all_kinds(self) -> list[str]:
+        if hasattr(self._schema, "inherit_from"):
+            return [self._schema.kind] + self._schema.inherit_from
+        return [self._schema.kind]
+
     def is_ip_prefix(self) -> bool:
         builtin_ipprefix_kind = "BuiltinIPPrefix"
         return self.get_kind() == builtin_ipprefix_kind or builtin_ipprefix_kind in self._schema.inherit_from  # type: ignore[union-attr]
@@ -948,7 +985,9 @@ class InfrahubNodeBase:
         for item in original_data.keys():
             if item in data.keys():
                 if data[item] == original_data[item]:
-                    data.pop(item)
+                    if attr := getattr(self, item, None):  # this should never be None, just a safety default value
+                        if not isinstance(attr, Attribute) or not attr.value_has_been_mutated:
+                            data.pop(item)
                     continue
                 if isinstance(original_data[item], dict):
                     self._strip_unmodified_dict(data=data, original_data=original_data, variables=variables, item=item)
@@ -1201,7 +1240,7 @@ class InfrahubNode(InfrahubNodeBase):
         else:
             await self._client.group_context.add_related_nodes(ids=[self.id], update_group_context=update_group_context)
 
-        self._client.store.set(key=self.id, node=self)
+        self._client.store.set(node=self)
 
     async def generate_query_data(
         self,
@@ -1418,8 +1457,10 @@ class InfrahubNode(InfrahubNodeBase):
     ) -> None:
         mutation_query = self._generate_mutation_query()
 
+        # Upserting means we may want to create, meaning payload contains all mandatory fields required for a creation,
+        # so hfid is just redondant information. Currently, upsert mutation has performance overhead if `hfid` is filled.
         if allow_upsert:
-            input_data = self._generate_input_data(exclude_hfid=False, request_context=request_context)
+            input_data = self._generate_input_data(exclude_hfid=True, request_context=request_context)
             mutation_name = f"{self._schema.kind}Upsert"
             tracker = f"mutation-{str(self._schema.kind).lower()}-upsert"
         else:
@@ -1477,15 +1518,15 @@ class InfrahubNode(InfrahubNodeBase):
         for rel_name in self._relationships:
             rel = getattr(self, rel_name)
             if rel and isinstance(rel, RelatedNode):
-                relation = node_data["node"].get(rel_name)
-                if relation.get("node", None):
+                relation = node_data["node"].get(rel_name, None)
+                if relation and relation.get("node", None):
                     related_node = await InfrahubNode.from_graphql(
                         client=self._client, branch=branch, data=relation, timeout=timeout
                     )
                     related_nodes.append(related_node)
             elif rel and isinstance(rel, RelationshipManager):
-                peers = node_data["node"].get(rel_name)
-                if peers:
+                peers = node_data["node"].get(rel_name, None)
+                if peers and peers["edges"]:
                     for peer in peers["edges"]:
                         related_node = await InfrahubNode.from_graphql(
                             client=self._client, branch=branch, data=peer, timeout=timeout
@@ -1724,7 +1765,7 @@ class InfrahubNodeSync(InfrahubNodeBase):
         else:
             self._client.group_context.add_related_nodes(ids=[self.id], update_group_context=update_group_context)
 
-        self._client.store.set(key=self.id, node=self)
+        self._client.store.set(node=self)
 
     def generate_query_data(
         self,
