@@ -1,5 +1,4 @@
 import os
-import shutil
 import subprocess  # noqa: S404
 from pathlib import Path
 
@@ -13,9 +12,10 @@ from .container import PROJECT_ENV_VARIABLES, InfrahubDockerCompose
 class TestInfrahubDocker:
     @pytest.fixture(scope="class")
     def infrahub_version(self) -> str:
-        return infrahub_version
+        return os.getenv("INFRAHUB_TESTING_IMAGE_VER") or infrahub_version
 
-    def execute_ctl_run(self, address: str, script: str) -> str:
+    @staticmethod
+    def execute_ctl_run(address: str, script: str) -> str:
         env = os.environ.copy()
         env["INFRAHUB_ADDRESS"] = address
         env["INFRAHUB_API_TOKEN"] = PROJECT_ENV_VARIABLES["INFRAHUB_TESTING_INITIAL_ADMIN_TOKEN"]
@@ -25,6 +25,17 @@ class TestInfrahubDocker:
         )
         return result.stdout
 
+    @staticmethod
+    def execute_command(command: str, address: str, concurrent_execution: int = 10) -> subprocess.CompletedProcess[str]:
+        env = os.environ.copy()
+        env["INFRAHUB_ADDRESS"] = address
+        env["INFRAHUB_API_TOKEN"] = PROJECT_ENV_VARIABLES["INFRAHUB_TESTING_INITIAL_ADMIN_TOKEN"]
+        env["INFRAHUB_MAX_CONCURRENT_EXECUTION"] = f"{concurrent_execution}"
+        result = subprocess.run(  # noqa: S602
+            command, shell=True, capture_output=True, text=True, env=env, check=False
+        )
+        return result
+
     @pytest.fixture(scope="class")
     def tmp_directory(self, tmpdir_factory: pytest.TempdirFactory) -> Path:
         directory = Path(str(tmpdir_factory.getbasetemp().strpath))
@@ -80,73 +91,3 @@ class TestInfrahubDocker:
     @pytest.fixture(scope="class")
     def task_manager_port(self, infrahub_app: dict[str, int]) -> int:
         return infrahub_app["task-manager"]
-
-    def backup_database(self, request: pytest.FixtureRequest, dest_dir: Path | None = None) -> None:
-        assert "enterprise" in os.environ.get("NEO4J_DOCKER_IMAGE", "")
-
-        backup_dir: Path = request.getfixturevalue("remote_backups_dir")
-        infrahub_compose: InfrahubDockerCompose = request.getfixturevalue("infrahub_compose")
-
-        infrahub_compose.exec_in_container(
-            command=[
-                "neo4j-admin",
-                "database",
-                "backup",
-                "--to-path",
-                os.environ.get(
-                    "INFRAHUB_TESTING_INTERNAL_DB_BACKUP_DIRECTORY",
-                    PROJECT_ENV_VARIABLES["INFRAHUB_TESTING_INTERNAL_DB_BACKUP_DIRECTORY"],
-                ),
-            ],
-            service_name="database",
-        )
-
-        if dest_dir:
-            shutil.copytree(
-                str(backup_dir),
-                str(dest_dir),
-            )
-
-    def restore_database(self, request: pytest.FixtureRequest, backup_file: Path) -> None:
-        assert "enterprise" in os.environ.get("NEO4J_DOCKER_IMAGE", "")
-
-        backup_dir: Path = request.getfixturevalue("remote_backups_dir")
-        infrahub_compose: InfrahubDockerCompose = request.getfixturevalue("infrahub_compose")
-
-        shutil.copy(
-            str(backup_file),
-            str(backup_dir / backup_file.name),
-        )
-
-        infrahub_compose.exec_in_container(
-            command=["cypher-shell", "-u", "neo4j", "-p", "admin", "STOP DATABASE neo4j;"],
-            service_name="database",
-        )
-
-        infrahub_compose.exec_in_container(
-            command=[
-                "neo4j-admin",
-                "database",
-                "restore",
-                "--overwrite-destination",
-                "--from-path",
-                str(
-                    Path(
-                        os.environ.get(
-                            "INFRAHUB_TESTING_INTERNAL_DB_BACKUP_DIRECTORY",
-                            PROJECT_ENV_VARIABLES["INFRAHUB_TESTING_INTERNAL_DB_BACKUP_DIRECTORY"],
-                        )
-                    )
-                    / backup_file.name
-                ),
-            ],
-            service_name="database",
-        )
-
-        infrahub_compose.exec_in_container(
-            command=["cypher-shell", "-d", "system", "-u", "neo4j", "-p", "admin", "START DATABASE neo4j;"],
-            service_name="database",
-        )
-
-        infrahub_compose.stop(down=False)
-        infrahub_compose.start()
